/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Main.java to edit this template
 */
package convexhull;

import java.io.File;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Scanner;
import java.util.Stack;

class CompareXY implements Comparator<Point> {

    @Override
    public int compare(Point o1, Point o2) {
        if(o1.getY() < o2.getY()) {
            return -1;
        } else if(o1.getY() > o2.getY()) {
            return 1;
        } else if(o1.getX() < o2.getX()) {
            return -1;
        } else if(o1.getX() > o2.getX()) {
            return 1;
        }
        return 0;
    }

}

class CompareAngle implements Comparator<Point> {
    private Point p0;
    
    public CompareAngle(Point p) {
        p0 = p;
    }

    @Override
    public int compare(Point o1, Point o2) {
        Vector v1 = new Vector(p0,o1);
        Vector v2 = new Vector(p0,o2);
        float angle1 = v1.polarAngle();
        float angle2 = v2.polarAngle();
        if(angle1 < angle2) {
            return -1;
        } else if(angle1 > angle2) {
            return 1;
        }
        return 0;
    }
}

public class ConvexHull {

    public static boolean isRightTurn(Point p0,Point p1,Point p2) {
        Vector v0 = new Vector(p0,p1);
        Vector v1 = new Vector(p0,p2);
        return Vector.crossProduct(v0,v1) < 0.0f;
    }
    
    public static void main(String[] args) {
        List<Point> points = new ArrayList<Point>();
        
        Scanner input;
        try {
            input = new Scanner(new File("points.txt"));
        } catch (Exception ex) {
            ex.printStackTrace();
            return;
        }
        while(input.hasNextFloat()) {
            float x = input.nextFloat();
            float y = input.nextFloat();
            Point p = new Point(x,y);
            points.add(p);
        }
        input.close();
        
        points.sort(new CompareXY());
        Point p0 = points.get(0);
        points.remove(0);
        points.sort(new CompareAngle(p0));
        
        Stack<Point> s = new Stack<Point>();
        s.push(p0);
        s.push(points.get(0));
        s.push(points.get(1));
        
        for(int n = 2;n < points.size();n++) {
            Point next = points.get(n);
            Point top = s.peek();
            s.pop();
            Point nextToTop = s.peek();
            s.push(top);
            while(isRightTurn(nextToTop,top,next)) {
                s.pop();
                top = s.peek();
                s.pop();
                nextToTop = s.peek();
                s.push(top);
            }
            s.push(next);
        }
        for(Point p : s) {
            p.print();
        }
    }
    
}
